package net.knarcraft.blockhunt.config;

import net.knarcraft.blockhunt.MemoryStorage;

/**
 * An identifier for a specific type of chat message
 */
public enum ChatMessageTypeIdentifier {

    /**
     * A normal chat message
     */
    NORMAL("%N", ConfigKey.CHAT_NORMAL),

    /**
     * A warning message
     */
    WARNING("%W", ConfigKey.CHAT_WARNING),

    /**
     * An error message
     */
    ERROR("%E", ConfigKey.CHAT_ERROR),

    /**
     * An argument which should be highlighted
     */
    ARGUMENT("%A", ConfigKey.CHAT_ARGUMENT),

    /**
     * A BlockHunt header
     */
    HEADER("%H", ConfigKey.CHAT_HEADER),

    /**
     * A tag specifying which plugin this message came from
     */
    TAG("%TAG", ConfigKey.CHAT_HEADER, ConfigKey.CHAT_TAG, ConfigKey.CHAT_NORMAL),
    ;

    private final String placeholder;
    private final ConfigKey[] configKeys;

    /**
     * Instantiates a new chat message type identifier
     *
     * @param placeholder <p>The placeholder used to specify this message type</p>
     * @param configKeys  <p>The configuration keys used to select this chat message type</p>
     */
    ChatMessageTypeIdentifier(String placeholder, ConfigKey... configKeys) {
        this.placeholder = placeholder;
        this.configKeys = configKeys;
    }

    /**
     * Replaces every instance of this chat message type identifier
     *
     * @param input <p>The input string to replace for</p>
     * @return <p>The input with this chat message type identifier replaced</p>
     */
    public String replace(String input) {
        StringBuilder replacement = new StringBuilder();
        for (ConfigKey key : configKeys) {
            replacement.append((String) MemoryStorage.config.get(key));
        }
        return input.replaceAll(this.placeholder, replacement.toString());
    }

    /**
     * Replaces every instance for every chat message type identifier
     *
     * @param input <p>The input string to replace for</p>
     * @return <p>The input with all chat message type identifiers replaced</p>
     */
    public static String replaceAll(String input) {
        for (ChatMessageTypeIdentifier identifier : ChatMessageTypeIdentifier.values()) {
            input = identifier.replace(input);
        }
        return input;
    }

}
